package logic

import (
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/net/ghttp"
	"github.com/gogf/gf/v2/os/gview"
	"github.com/gogf/gf/v2/text/gstr"
	cmsService "github.com/tiger1103/gfast/v3/internal/app/cms/service"
	"github.com/tiger1103/gfast/v3/internal/app/home/service"
	"github.com/tiger1103/gfast/v3/library/libUtils"
	"reflect"
	"unsafe"
)

func init() {
	service.RegisterCmsTemplate(NewCmsTemplateService())
}

func NewCmsTemplateService() *cmsTemplateService {
	return &cmsTemplateService{}
}

type cmsTemplateService struct{}

func (s *cmsTemplateService) GetCmsView(r *ghttp.Request, singleton ...bool) (v *gview.View) {
	cmsConfigMap, err := cmsService.CmsConfig().GetCmsConfigMap(r.GetCtx())
	if err != nil {
		r.Response.WriteExit(err.Error())
	}
	themeName := "default"
	if _, ok := cmsConfigMap["theme_template"]; ok {
		themeName = cmsConfigMap["theme_template"]
	}
	//获取CMS配置信息
	if len(singleton) > 0 && singleton[0] {
		v = gview.Instance()
		v.AddPath("./resource/template/cms/theme/" + themeName)
	} else {
		v = gview.New("./resource/template/cms/theme/" + themeName)
	}
	//v.SetAutoEncode(true)
	delimiters, err := g.Cfg().Get(r.GetCtx(), "viewer.delimiters", []string{"${", "}"})
	defaultFile, err := g.Cfg().Get(r.GetCtx(), "viewer.defaultFile", "index.html")
	v.SetConfigWithMap(g.Map{
		"delimiters":  delimiters.Array(),
		"defaultFile": defaultFile.String(),
	})
	// 设置当前主题皮肤路径
	v.Assign("cmsTheme", "/cms/"+themeName)
	domain := libUtils.GetDomain(r.GetCtx())
	v.Assign("domain", domain)
	// CMS配置对象
	v.Assign("cmsConfig", cmsConfigMap)
	v.Assign("Request", r.Request)
	v.Assign("Session", r.Session.MustData())
	v.Assign("Query", r.GetQueryMap())
	v.Assign("Form", r.GetFormMap())
	// 默认SEO信息
	v.Assign("seoTitle", cmsConfigMap["seo_title"])
	v.Assign("seoKeywords", cmsConfigMap["seo_keywords"])
	v.Assign("seoDescription", cmsConfigMap["seo_description"])
	// 绑定字符串处理函数
	s.bindTextFunc(v)
	//绑定模板中需要用到的方法
	err = service.CmsTemplateTags().BindFunMap(v)
	if err != nil {
		r.Response.WriteExit(err.Error())
	}
	return
	//文章参数
}

// 绑定gstr包中的函数到模板中
func (s *cmsTemplateService) bindTextFunc(v *gview.View) {
	v.BindFuncMap(g.Map{
		"AddSlashes":      gstr.AddSlashes,
		"Chr":             gstr.Chr,
		"ChunkSplit":      gstr.ChunkSplit,
		"Compare":         gstr.Compare,
		"Count":           gstr.Count,
		"CountChars":      gstr.CountChars,
		"CountI":          gstr.CountI,
		"CountWords":      gstr.CountWords,
		"Equal":           gstr.Equal,
		"Explode":         gstr.Explode,
		"Fields":          gstr.Fields,
		"HasPrefix":       gstr.HasPrefix,
		"HasSuffix":       gstr.HasSuffix,
		"HideStr":         gstr.HideStr,
		"Implode":         gstr.Implode,
		"InArray":         gstr.InArray,
		"IsLetterLower":   gstr.IsLetterLower,
		"IsLetterUpper":   gstr.IsLetterUpper,
		"IsNumeric":       gstr.IsNumeric,
		"Join":            gstr.Join,
		"JoinAny":         gstr.JoinAny,
		"LcFirst":         gstr.LcFirst,
		"LenRune":         gstr.LenRune,
		"Nl2Br":           gstr.Nl2Br,
		"NumberFormat":    gstr.NumberFormat,
		"Ord":             gstr.Ord,
		"QuoteMeta":       gstr.QuoteMeta,
		"Repeat":          gstr.Repeat,
		"Replace":         gstr.Replace,
		"ReplaceByArray":  gstr.ReplaceByArray,
		"ReplaceByMap":    gstr.ReplaceByMap,
		"ReplaceI":        gstr.ReplaceI,
		"ReplaceIByArray": gstr.ReplaceIByArray,
		"ReplaceIByMap":   gstr.ReplaceIByMap,
		"Reverse":         gstr.Reverse,
		"SearchArray":     gstr.SearchArray,
		"Shuffle":         gstr.Shuffle,
		"Split":           gstr.Split,
		"SplitAndTrim":    gstr.SplitAndTrim,
		"Str":             gstr.Str,
		"StrEx":           gstr.StrEx,
		"StripSlashes":    gstr.StripSlashes,
		"StrLimit":        gstr.StrLimit,
		"StrLimitRune":    gstr.StrLimitRune,
		"StrTill":         gstr.StrTill,
		"StrTillEx":       gstr.StrTillEx,
		"SubStr":          gstr.SubStr,
		"SubStrRune":      gstr.SubStrRune,
		"ToLower":         gstr.ToLower,
		"ToUpper":         gstr.ToUpper,
		"UcFirst":         gstr.UcFirst,
		"UcWords":         gstr.UcWords,
		"WordWrap":        gstr.WordWrap,
	})

}

// GetViewData 获取view 的 data对象
func (s *cmsTemplateService) GetViewData() (data map[string]interface{}) {
	view := gview.Instance()
	viewRef := reflect.ValueOf(view).Elem()
	d := viewRef.FieldByName("data")
	d = reflect.NewAt(d.Type(), unsafe.Pointer(d.UnsafeAddr())).Elem()
	data = d.Interface().(map[string]interface{})
	return
}
